/*! \file tfrfme.h
 *
 * \brief Representation of the trapping configuration objects.
 */

#ifndef INCLUDE_TFRFME_H_
#define INCLUDE_TFRFME_H_

/*! \brief Class to represent the trapping configuration.
 */
class TFRFME {
 private:
  //! NLMMT = 2 * LM * (LM + 2)
  int nlmmt;
  //! NRVC = NXV * NYV * NZV
  int nrvc;
  
 protected:
  //! Field expansion mode identifier.
  int lmode;
  //! Maximim field expansion order;
  int lm;
  //! QUESTION: definition?
  int nkv;
  //! Number of computed X coordinates.
  int nxv;
  //! Number of computed Y coordinates.
  int nyv;
  //! Number of computed Z coordinates.
  int nzv;
  //! Wave number in scale units
  double vk;
  //! External medium refractive index
  double exri;
  //! QUESTION: definition?
  double an;
  //! QUESTION: definition?
  double ff;
  //! QUESTION: definition?
  double tra;
  //! QUESTION: definition?
  double spd;
  //! QUESTION: definition?
  double frsh;
  //! QUESTION: definition?
  double exril;
  //! Vector of computed x positions
  double *xv;
  //! Vector of computed y positions
  double *yv;
  //! Vector of computed z positions
  double *zv;
  //! QUESTION: definition?
  complex<double> **wsum;

 public:
  /*! \briesf Trapping configuration instance constructor.
   */
  TFRFME(
	 int _lmode, int _lm, int _nkv, int _nxv, int _nyv, int _nzv
);
  
  /*! \briesf Trapping configuration instance destroyer.
   */
  ~TFRFME();

  /*! \briesf Load a trapping configuration instance from binary file.
   *
   * \param file_name: `string` Name of the file.
   * \param format: `string` Format of the file (can be either "HDF5"
   * or "LGEACY". Default is "LEGACY").
   * \return instance: `TFRFME *` Pointer to anewly created configuration
   * instance.
   */
  TFRFME* from_binary(std::string file_name, std::string format="LEGACY");

  /*! \brief Get an element from the WSUM matrix.
   *
   * \param row: `int` Row index of the element to be read.
   * \param col: `int` Column index of the element to be read.
   * \return value: `complex<double>` The value of the requested element.
   */
  std::complex<double> get_matrix_element(int row, int col);

  /*! \brief Get a configuration parameter.
   *
   * \param param_name: `string` Name of the parameter.
   * \return value: `double` The value of the requested parameter.
   */
  double get_param(std::string param_name);

  /*! \brief Set an element in the WSUM matrix.
   *
   * \param row: `int` Row index of the element to be read.
   * \param col: `int` Column index of the element to be read.
   * \param value: `complex<double>` The value to be placed in the matrix.
   */
  void set_matrix_element(int row, int col, std::complex<double> value);
  
  /*! \brief Set a configuration parameter.
   *
   * \param param_name: `string` Name of the parameter.
   * \param value: `double` Value to be stored as parameter.
   */
  void set_param(std::string param_name, double value);

  /*! \briesf Write a trapping configuration instance to binary file.
   *
   * \param file_name: `string` Name of the file.
   * \param format: `string` Format of the file (can be either "HDF5"
   * or "LGEACY". Default is "LEGACY").
   */
  void write_binary(std::string file_name, std::string format="LEGACY");
};
#endif
